#!/usr/bin/env python

import abc

class ErrorHandler(object):
    __metaclass__ = abc.ABCMeta

    @abc.abstractmethod
    def check(self):
        pass

    @abc.abstractmethod
    def correct(self):
        pass




#!/usr/bin/env python

'''
Created on May 2, 2012
'''

from __future__ import division

__author__ = "Shyue Ping Ong"
__copyright__ = "Copyright 2012, The Materials Project"
__version__ = "0.1"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyue@mit.edu"
__date__ = "May 2, 2012"

import abc


class Job(object):
    __metaclass__ = abc.ABCMeta

    @abc.abstractmethod
    def setup(self):
        """
        Allows for pre-processing steps.
        """
        pass

    @abc.abstractmethod
    def run(self):
        """
        Actual work done.
        """
        pass

    @abc.abstractmethod
    def postprocess(self):
        """
        Post-processing after a job. Can be a cleanup, analysis of results, or
        anything actually.
        """
        pass

    @abc.abstractproperty
    def name(self):
        """
        A nice string name for the job.
        """
        pass




#!/usr/bin/env python

'''
Created on May 2, 2012
'''

from __future__ import division

__author__ = "Shyue Ping Ong"
__copyright__ = "Copyright 2012, The Materials Project"
__version__ = "0.1"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyue@mit.edu"
__date__ = "May 2, 2012"

import logging
import os
import tarfile


class Custodian(object):

    def __init__(self, handlers, jobs, max_errors=1):
        """
        Args:
            handlers:
                Error handlers. In order of priority of fixing.
            jobs:
                List of Jobs. Allow for multistep jobs. E.g., give it two
                BasicVaspJobs and you effectively have a aflow
                double-relaxation.
            max_errors:
                Maximum number of errors allowed before exiting.
        """
        self.max_errors = max_errors
        self.jobs = jobs
        self.handlers = handlers

    def run(self):
        total_errors = 0
        for i, job in enumerate(self.jobs):
            attempt = 0
            while total_errors < self.max_errors:
                logging.info("Starting job no. {} ({}) attempt no. {}. Errors thus far = {}.".format(i + 1, job.name, attempt + 1, total_errors))
                job.setup()
                job.run()
                error = False
                for h in self.handlers:
                    if h.check():
                        logging.error(str(h))
                        self.backup(i, attempt)
                        h.correct()
                        total_errors += 1
                        attempt += 1
                        error = True
                        break
                job.postprocess()
                if not error:
                    break
        if total_errors == self.max_errors:
            logging.info("Max {} errors reached. Exited".format(total_errors))
        else:
            logging.info("Run completed")

    def backup(self, job_no, attempt_no):
        filename = "job_{}_attempt_{}.tar.gz".format(job_no + 1,
                                                     attempt_no + 1)
        logging.info("Backing up run to {}.".format(filename))
        tar = tarfile.open(filename, "w:gz")
        for f in os.listdir("."):
            tar.add(f)
        tar.close()




#!/usr/bin/env python

"""
TODO: Change the module doc.
"""

from __future__ import division

__author__ = "Shyue Ping Ong"
__version__ = "0.1"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyuep@gmail.com"
__status__ = "Beta"
__date__ = "2/4/13"


import shutil

from custodian.handlers import ErrorHandler
from pymatgen.io.vaspio.vasp_input import Incar, Kpoints, Poscar
from pymatgen.io.vaspio.vasp_output import Vasprun


class IncarErrorHandler(ErrorHandler):

    def check(self):
        with open("vasp.out", "r") as f:
            output = f.read()
            for line in output.split("\n"):
                l = line.strip()
                if l.startswith("Tetrahedron method fails for NKPT<4") or l.strip().startswith("Fatal error detecting k-mesh") or l.strip().startswith("Fatal error: unable to match k-point"):
                    self.error = "tet"
                    return True
                elif l.startswith("inverse of rotation matrix was not found (increase SYMPREC)"):
                    self.error = "inv_rot_mat"
                    return True
                elif l.startswith("BRMIX: very serious problems"):
                    self.error = "brmix"
                    return True
                elif l.startswith("WARNING: Sub-Space-Matrix is not hermitian in DAV"):
                    self.error = "subspacematrix"
                    return True
        return False

    def correct(self):
        shutil.copy("INCAR", "INCAR.orig")
        incar = Incar.from_file("INCAR")
        if self.error == "tet":
            incar['ISMEAR'] = 0
        elif self.error == "inv_rot_mat":
            incar['SYMPREC'] = 1e-8
        elif self.error == "brmix":
            incar['IMIX'] = 1
        elif self.error == "subspacematrix":
            incar['LREAL'] = False
        incar.write_file("INCAR")

    def __str__(self):
        return "INCAR error"


class KpointsErrorHandler(ErrorHandler):

    def check(self):
        with open("vasp.out", "r") as f:
            output = f.read()
            for line in output.split("\n"):
                l = line.strip()
                if l.startswith("Routine TETIRR needs special values"):
                    self.error = "tetirr"
                    return True
                elif l.startswith("Could not get correct shifts"):
                    self.error = "incorrect_shift"
                    return True
        return False

    def correct(self):
        shutil.copy("KPOINTS", "KPOINTS.orig")
        kpoints = Kpoints.from_file("KPOINTS")
        if self.error == "tetirr":
            kpoints.style = "Gamma"
        elif self.error == "incorrect_shift":
            kpoints.style = "Gamma"
        kpoints.write_file("KPOINTS")

    def __str__(self):
        return "KPOINTS error"


class UnconvergedErrorHandler(ErrorHandler):

    def check(self):
        v = Vasprun('vasprun.xml')
        if not v.converged:
            return True
        return False

    def correct(self):
        shutil.copy("CONTCAR", "POSCAR")
        incar = Incar.from_file("INCAR")
        incar['ISTART'] = 1
        incar.write_file("INCAR")

    def __str__(self):
        return "Run unconverged."


class PoscarErrorHandler(ErrorHandler):

    def check(self):
        with open("vasp.out", "r") as f:
            output = f.read()
            for line in output.split("\n"):
                l = line.strip()
                if l.startswith("Found some non-integer element in rotation matrix"):
                    return True
        return False

    def correct(self):
        #TODO: Add transformation applied to transformation.json if exists.
        from pymatgen.transformations.standard_transformations import PerturbStructureTransformation
        shutil.copy("POSCAR", "POSCAR.orig")
        p = Poscar.from_file("POSCAR")
        s = p.struct
        trans = PerturbStructureTransformation(0.05)
        new_s = trans.apply_transformation(s)
        p = Poscar(new_s)
        p.write_file("POSCAR")


"""
Aflow Error 5 (classrotmat)

grep : "Reciprocal lattice and k-lattice belong"
fix : SYMPREC = 1E-10, KPOINTS (equilize mesh)

Aflow Error 9 (davidson)

grep : "WARNING: Sub-Space-Matrix is not hermitian in DAV"
fix : INCAR -> remove ALGO, IMIX, set ALGO=VERY_FAST

Aflow Error 10 (nbands)

grep : "BRMIX: very serious problems"
fix :
notes:
aflow tries to get NBANDS from the OUTCAR
-> nbands=atoi(word.substr(word.find("NBANDS")+7).c_str());
aflow then tries to get NBANDS from INCAR
Then it increases NBANDS via
nbands=nbands+5+nbands/5

"""



#!/usr/bin/env python

"""
TODO: Change the module doc.
"""

from __future__ import division

__author__ = "Shyue Ping Ong"
__version__ = "0.1"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyuep@gmail.com"
__status__ = "Beta"
__date__ = "2/4/13"


import subprocess
import os
import shutil

from pymatgen.util.io_utils import zopen
from pymatgen.io.vaspio.vasp_input import Incar, Poscar
from pymatgen.io.cifio import CifParser
from pymatgen.io.vaspio_set import MITVaspInputSet

from custodian.jobs import Job


class BasicVaspJob(Job):
    """
    Very basic vasp job. Just runs whatever is in the directory. But
    conceivably can be a complex processing of inputs etc. with initialization.
    """

    def __init__(self, default_vasp_input_set=MITVaspInputSet()):
        self.default_vis = default_vasp_input_set

    def setup(self):
        files = os.listdir(".")
        num_structures = 0
        if not set(files).issuperset(set(["INCAR", "POSCAR", "POTCAR", "KPOINTS"])):
            for f in files:
                if f.startswith("POSCAR") or f.startswith("CONTCAR"):
                    poscar = Poscar.from_file(f)
                    struct = poscar.struct
                    num_structures += 1
                elif f.lower().endswith(".cif"):
                    parser = CifParser(f)
                    struct = parser.get_structures()[0]
                    num_structures += 1
            if num_structures != 1:
                raise RuntimeError("{} structures found. Unable to continue.")
            else:
                self.default_vis.write_input(struct, ".")

    def run(self):
        args = ["mpirun", "/share/apps/bin/pvasp.5.2.11"]
        output = subprocess.Popen(args, stdout=subprocess.PIPE).communicate()[0]
        with open("vasp.out", 'w') as f:
            f.write(output)

    def postprocess(self):
        pass

    @property
    def name(self):
        return "Basic Vasp Job"


class SecondRelaxationVaspJob(BasicVaspJob):
    """
    Very basic vasp job. Just runs whatever is in the directory. But
    conceivably can be a complex processing of inputs etc. with initialization.
    """

    OUTPUT_FILES = ['DOSCAR', 'INCAR', 'KPOINTS', 'POSCAR', 'PROCAR',
                    'vasprun.xml', 'CHGCAR', 'CHG', 'EIGENVAL', 'OSZICAR',
                    'WAVECAR', 'CONTCAR', 'IBZKPT', 'OUTCAR', 'vasp.out']

    def setup(self):
        for f in SecondRelaxationVaspJob.OUTPUT_FILES:
            shutil.copy(f, "{}.relax1".format(f))
        shutil.copy("CONTCAR", "POSCAR")
        incar = Incar.from_file("INCAR")
        incar['ISTART'] = 1
        incar.write_file("INCAR")

    def postprocess(self):
        for f in SecondRelaxationVaspJob.OUTPUT_FILES:
            shutil.copy(f, "{}.relax2".format(f))
        for f in os.listdir("."):
            if not f.endswith("gz"):
                with zopen(f, 'rb') as f_in, zopen('{}.gz'.format(f), 'wb') as f_out:
                    f_out.writelines(f_in)
                os.remove(f)

    @property
    def name(self):
        return "Second relaxation Vasp Job"




import os

from distribute_setup import use_setuptools
use_setuptools(version='0.6.10')
from setuptools import setup, find_packages, Extension

with open("README.rst") as f:
    long_desc = f.read()
    
setup(
    name="custodian",
    packages=find_packages(),
    version="0.1.0a",
    install_requires=[],
    extras_require={"vasp": ["pymatgen>=2.4.3"]},
    package_data={},
    author="Shyue Ping Ong",
    author_email="shyuep@gmail.com",
    maintainer="Shyue Ping Ong",
    url="https://github.com/materialsproject/custodian",
    license="MIT",
    description="A simple JIT job management framework in Python.",
    long_description=long_desc,
    keywords=["jit", "just-in-time", "job", "management", "vasp"],
    classifiers=[
        "Programming Language :: Python :: 2.7",
        "Development Status :: 4 - Beta",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Scientific/Engineering :: Information Analysis",
        "Topic :: Scientific/Engineering :: Physics",
        "Topic :: Scientific/Engineering :: Chemistry",
        "Topic :: Software Development :: Libraries :: Python Modules"
    ],
    download_url="https://github.com/materialsproject/custodian/archive/master.zip",
    scripts=[os.path.join("scripts", f) for f in os.listdir("scripts")]
)



#!/usr/bin/env python

"""
TODO: Change the module doc.
"""

from __future__ import division

__author__ = "shyuepingong"
__version__ = "0.1"
__maintainer__ = "Shyue Ping Ong"
__email__ = "shyuep@gmail.com"
__status__ = "Beta"
__date__ = "2/4/13"

import logging

from custodian.custodian import Custodian
from pymatpro.custodian.handlers import IncarErrorHandler,\
    KpointsErrorHandler, UnconvergedErrorHandler
from pymatpro.custodian.jobs import BasicVaspJob, SecondRelaxationVaspJob


def aflow_run():
    #An example of an aflow run.

    FORMAT = '%(asctime)s %(message)s'
    logging.basicConfig(format=FORMAT, level=logging.INFO, filename="run.log")
    handlers = [IncarErrorHandler(), KpointsErrorHandler(),
                UnconvergedErrorHandler()]
    jobs = [BasicVaspJob(), SecondRelaxationVaspJob()]
    c = Custodian(handlers, jobs, max_errors=10)
    c.run()

if __name__ == "__main__":
    aflow_run()